import importlib

import pkg_resources

from pkg_scripts.misc_functions import open_database, check_if_exists, install, get_version, update_requirements_file
from pkg_scripts.pkg_updation import update_packages


def add_dependency(conn, name, parent_pid, db):
    dependency_package_insert_query = db.insert().values(
        name=name,
        version=get_version(name),
        parent_id=parent_pid
    )

    conn.execute(dependency_package_insert_query)


def add_package(conn, package, package_name, db):
    print("Installing: " + package)
    install(package)
    primary_package_insert_query = db.insert().values(
        name=package_name,
        version=get_version(package),
        parent_id=None
    )

    result = conn.execute(primary_package_insert_query)

    return result.inserted_primary_key[0]


# Function to retrieve a list of the dependencies of the package
def get_dependencies(package_name):
    importlib.reload(pkg_resources)
    _package = pkg_resources.working_set.by_key[package_name]
    print("Looking for dependencies of: ")
    print(_package)
    return [str(r) for r in _package.requires()]


def perform_add_module(conn, packages_to_install, db):
    for package in packages_to_install:
        if "==" in package:
            package_name = package[:package.index("=")]
            version = package[package.index("==") + 2:]
        else:
            package_name = package
            version = None
        print(package_name)
        print(version)
        exists = check_if_exists(conn, package_name, version=None, db=db)
        if len(exists) == 1:
            update_packages(packages_to_update=[package], db=db, conn=conn)

        if len(exists) == 0:
            pid = add_package(conn, package, package_name, db)

            dependencies = get_dependencies(package_name)
            for dep in dependencies:
                add_dependency(conn, dep, pid, db)

        else:
            print("Package already installed")
            return False


def install_packages(packages_to_install, db, engine):
    conn = open_database(engine)
    perform_add_module(conn, packages_to_install, db)
    update_requirements_file(conn, db)


def install_requirements(db, engine):
    requirements_file = open('requirements.txt', 'r')
    packages = requirements_file.readlines()
    install_packages(packages, db, engine)



from pkg_scripts.misc_functions import check_if_exists, install, get_version, update_requirements_file


def update_packages(packages_to_update, db, conn):
    for package in packages_to_update:
        if '==' in package:
            package_name = package[:package.index("=")]
        else:
            package_name = package

        pid = check_if_exists(conn, package_name, version=None, db=db)
        print("pid = ")
        print(pid)
        if pid is not None:
            install(package)
            update_package_query = db.update().values(version=get_version(package_name)).where(db.c.name == package_name)
            conn.execute(update_package_query)
        
            return True
        
        else:
            return False
    
    update_requirements_file(conn, db)




from pathlib import Path
import sys

from pkg_scripts.db_management import Database
from pkg_scripts.pkg_installation import install_packages, install_requirements
from pkg_scripts.pkg_updation import update_packages
from pkg_scripts.pkg_uninstallation import uninstall_packages





def run_application():
    dbfile = Path("packages.db")
    database = Database()
    engine = database.engine
    db = database.packages

    if not dbfile.is_file():
        db, engine = database.initiate_engine()

    command = sys.argv[1]
    packages = sys.argv[2:]

    if command == "install" or command == "i":
        install_packages(packages, db, engine)

    elif command == "req" or command == 'r':
        install_requirements(db, engine)

    elif command == "uninstall" or command == "un":
        uninstall_packages(packages, db, engine)

    elif command == "update" or command == "up":
        update_packages(packages, db, engine)

    elif command == "help" or command == "h" or command == "?" or command is None:
        print("[+] Usage instructions:")
        print("""

    + Syntax:
    
    keepr <command> <package list>
    
    
    + Commands:
    
    * install - Install Packages
    * req - Install from requirements.txt
    * uninstall - Uninstall Packages and dependencies
    * update - Update an existing package
    * help - Display Help information
    * credits - List author credits
    
    
    + Example:    
    
    keepr install django==2.2 pymongo==1.2
    OR
    keepr req
            """)

    elif command == "credits" or command == "c":
        print(
            "This application was developed by:",
            "Sameeran Bandishti [sameeranbandishti@ieee.org]",
            "Ayush Priya [ayushpriya10@ieee.org]\n",
            "For any help or queries about the application, please contact the team at shopkeepr3.6@gmail.com",
            sep='\n'
        )

    else:
        print('[-] %s is not a command. Please use "help" to look at usage instructions.' % command)


if __name__ == "__main__":
    run_application()



import importlib
import subprocess

import pkg_resources
from sqlalchemy import select, and_


def open_database(engine):
    conn = engine.connect()
    print("Database Opened")

    return conn


def install(package):  # Function to install the given packaged
    subprocess.run(["pip", "install", package])


def uninstall(package):
    subprocess.run(["pip", "uninstall", package, "-y"])


def delete_package(conn, package, pid, db):
    print(pid)
    query = db.delete().where(db.c.pid == pid)
    conn.execute(query)
    uninstall(package)


def check_if_exists(conn, package_name, version, db):
    if version is not None:
        package_exists = select([db.c.pid]).where(and_(db.c.name == package_name, db.c.version == version))
    else:
        package_exists = select([db.c.pid]).where(and_(db.c.name == package_name))

    result = conn.execute(package_exists)
    print("results of check if exists query")
    print(result)
    pid_list=[]
    for _row in result:
        print(_row[0])
        pid_list.append(_row[0])

    return pid_list


def get_version(package):
    if "==" in package:
        print("Found ==  in package name")
        version = package[package.index("==")+2:]
        return version
    else:
        importlib.reload(pkg_resources)
        return pkg_resources.get_distribution(package).version


def update_requirements_file(conn, db):
    result = conn.execute(select([db.c.name, db.c.version]))
    packages = []
    print("Result is")
    print(result)
    for _row in result:
        print(_row[0])
        packages.append([_row[0], _row[1]])

    string = ""

    for val in packages:
        if val[1]:
            string += val[0] + "==" + val[1]
        else:
            string += val[0]

        string += "\n"
    print("String = ")
    print(string)
    requirements_file = open('requirements.txt', 'w')
    requirements_file.write(string)
    requirements_file.close()



from pkg_scripts.misc_functions import open_database, check_if_exists, uninstall, delete_package, update_requirements_file


def delete_dependencies(conn, parent_pid, db):
    select_dependencies = db.select().where(db.c.parent_id == parent_pid)
    result = conn.execute(select_dependencies)
    print("Selected Dependencies:")
    # for _row in result:
    #     print(_row)

    flag = True

    for _row in result:
        common_dependencies = conn.execute(db.select().where(db.c.name == _row[1]))
        print(f"List of entries for the dependency {_row[1]}")
        # for _com_dep in  common_dependencies:
        #     print(_com_dep)

        for _com_dep in common_dependencies:
            if int(_com_dep[3]) != parent_pid:
                print(str(_com_dep[3]) + " " + str(parent_pid))
                flag = False

        if flag:
            print("uninstalling:")
            print(_row[1])
            uninstall(_row[1])

        delete_dependency = db.delete().where(db.c.pid == _row[0])
        conn.execute(delete_dependency)
        flag = True


def perform_remove_module(conn, packages_to_uninstall, db):
    for package in packages_to_uninstall:
        if '==' in package:
            package_name = package[:package.index('=')]
        else:
            package_name = package
        parent_pid_list = check_if_exists(conn, package_name, version=None, db=db)
        print("Pid List")
        print(parent_pid_list)
        if parent_pid_list is not None:
            for parent_pid in parent_pid_list:
                delete_dependencies(conn, parent_pid, db)
                delete_package(conn, package, parent_pid, db)
        else:
            print("Package has not been installed")


def uninstall_packages(packages_to_uninstall, db, engine):
    conn = open_database(engine)
    print("Connection Established with database")
    perform_remove_module(conn, packages_to_uninstall, db)
    print("Module Removed")
    update_requirements_file(conn, db)
    print("Update Requirements")



from sqlalchemy import MetaData
from sqlalchemy import Table, Column, Integer, String
from sqlalchemy import create_engine


class Database():
    engine = create_engine(f'sqlite:///packages.db', echo=True)
    meta = MetaData()
    packages = Table(
        'packages', meta,
        Column('pid', Integer, primary_key=True, autoincrement=True),
        Column('name', String),
        Column('version', String, nullable=True),
        Column('parent_id', String, nullable=True)
    )

    def initiate_engine(self):
        self.meta.create_all(self.engine)

        return self.packages, self.engine


if __name__ == '__main__':
    initiate_engine()




import codecs
import os
import re

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

REQUIREMENTS = ['sqlalchemy']




try:
    fh = codecs.open("DESCRIPTION.md", encoding="utf-8")
    long_description = fh.read()
    fh.close()
except FileNotFoundError:
    long_description = ""

setup(
        name="shopkeepr",
        version="1.0.15",
        description="A command line tool for project management. Made for Developers",
        long_description=long_description,
        url="https://github.com/ayushpriya10/ShopKeepr",
        author="Sameeran Bandishti, Ayush Priya",
        author_email="shopkeepr3.6@gmail.com",
        packages=find_packages(include=[
            "pkg_scripts"
        ]),
        include_package_data=True,
        entry_points={
            "console_scripts": [
                'keepr = pkg_scripts.main:run_application'
            ]
        },
        classifiers=[
            "Development Status :: 4 - Beta",
            "License :: OSI Approved :: MIT License ",
            "Programming Language :: Python :: 3.6",
        ],
        install_requires=REQUIREMENTS,
        keywords='pip requirements automation'
)